<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\LoggerFactory;

use Monolog\Handler\AbstractHandler;
use Monolog\Handler\AbstractProcessingHandler;
use Monolog\Handler\BufferHandler;
use Monolog\Handler\DeduplicationHandler;
use Monolog\Handler\FingersCrossed\ErrorLevelActivationStrategy;
use Monolog\Handler\FingersCrossedHandler;
use Monolog\Handler\HandlerInterface;
use Monolog\Logger;

trait SpecialHandlersTrait
{
    /**
     * @param AbstractProcessingHandler $handler
     * @param array<int|string, string|array<string, string|int>> $specialHandlerFlags
     * @return HandlerInterface
     */
    public function applySpecialHandlers(
        AbstractProcessingHandler $handler,
        array $specialHandlerFlags = []
    ): HandlerInterface {
        $handler = $this->applyBufferHandler($specialHandlerFlags, $handler);
        $handler = $this->applyLogOnErrorOnlyHandler($specialHandlerFlags, $handler);
        return $this->applyMakeUniqueHandler($specialHandlerFlags, $handler);
    }

    public function setBuffering(
        HandlerInterface $handler,
        int $bufferLimit = 0,
        int $loglevel = Logger::DEBUG
    ): BufferHandler {
        /**  @phpstan-ignore argument.type */
        return new BufferHandler($handler, $bufferLimit, $loglevel);
    }

    public function setLogItemsOnErrorOnly(
        HandlerInterface $handler,
        int $activationLevel = Logger::ERROR
    ): FingersCrossedHandler {
        return new FingersCrossedHandler(
            $handler,
            /**  @phpstan-ignore argument.type */
            new ErrorLevelActivationStrategy($activationLevel)
        );
    }

    public function makeUnique(
        HandlerInterface $handler,
        int $deduplicationLevel = Logger::ERROR,
        int $time = 60
    ): DeduplicationHandler {
        /**  @phpstan-ignore argument.type */
        return new DeduplicationHandler($handler, null, $deduplicationLevel, $time);
    }

    /**
     * @param array<int|string, string|array<string, string|int>> $specialHandlerFlags
     * @param HandlerInterface $handler
     *
     * @return HandlerInterface
     */
    protected function applyBufferHandler(array $specialHandlerFlags, HandlerInterface $handler): HandlerInterface
    {
        if (in_array(self::SPECIAL_HANDLERS_BUFFERING, $specialHandlerFlags, true)) {
            $handler = $this->setBuffering($handler);
        } elseif (in_array(self::SPECIAL_HANDLERS_BUFFERING, array_keys($specialHandlerFlags), true)) {
            $options = $specialHandlerFlags[self::SPECIAL_HANDLERS_BUFFERING];
            $handler = $this->setBuffering(
                $handler,
                /**  @phpstan-ignore argument.type */
                $options[self::BUFFERING_OPTION_LIMIT] ?? 0,
                /**  @phpstan-ignore argument.type */
                $options[self::BUFFERING_OPTION_LEVEL] ?? Logger::DEBUG
            );
        }
        return $handler;
    }

    /**
     * @param array<int|string, string|array<string, string|int>> $specialHandlerFlags
     * @param HandlerInterface $handler
     *
     * @return HandlerInterface
     */
    protected function applyLogOnErrorOnlyHandler(array $specialHandlerFlags, HandlerInterface $handler): HandlerInterface
    {
        if (in_array(self::SPECIAL_HANDLERS_LOG_ON_ERROR_ONLY, $specialHandlerFlags, true)) {
            $handler = $this->setLogItemsOnErrorOnly($handler);
        } elseif (in_array(self::SPECIAL_HANDLERS_LOG_ON_ERROR_ONLY, array_keys($specialHandlerFlags), true)) {
            $options = $specialHandlerFlags[self::SPECIAL_HANDLERS_LOG_ON_ERROR_ONLY];
            $handler = $this->setLogItemsOnErrorOnly(
                $handler,
                /**  @phpstan-ignore argument.type */
                $options[self::LOGONERRORONLY_LEVEL] ?? Logger::ERROR
            );
        }
        return $handler;
    }

    /**
     * @param array<int|string, string|array<string, string|int>> $specialHandlerFlags
     * @param HandlerInterface $handler
     *
     * @return HandlerInterface
     */
    protected function applyMakeUniqueHandler(array $specialHandlerFlags, HandlerInterface $handler): HandlerInterface
    {
        if (in_array(self::SPECIAL_HANDLERS_MAKE_UNIQUE, $specialHandlerFlags, true)) {
            /**  @phpstan-ignore argument.type */
            $handler = $this->makeUnique($handler);
        } elseif (in_array(self::SPECIAL_HANDLERS_MAKE_UNIQUE, array_keys($specialHandlerFlags), true)) {
            $options = $specialHandlerFlags[self::SPECIAL_HANDLERS_MAKE_UNIQUE];
            $handler = $this->makeUnique(
                /**  @phpstan-ignore argument.type */
                $handler,
                /**  @phpstan-ignore argument.type */
                $options[self::MAKEUNIQUE_OPTION_LEVEL] ?? Logger::ERROR,
                /**  @phpstan-ignore argument.type */
                $options[self::MAKEUNIQUE_OPTION_TIME] ?? 60
            );
        }
        return $handler;
    }
}
