<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

namespace D3\LoggerFactory\tests;

use D3\LoggerFactory\SensitiveFilterProcessor;
use Generator;
use ReflectionException;

/**
 * @coversNothing
 */
class SensitiveFilterProcessorTest extends ApiTestCase
{
    /**
     * @test
     * @param string|null $replacementArgument
     * @param string      $expectedReplacement
     *
     * @return void
     * @throws ReflectionException
     * @covers \D3\LoggerFactory\SensitiveFilterProcessor::__construct
     * @dataProvider constructDataProvider
     */
    public function testConstruct(?string $replacementArgument, string $expectedReplacement)
    {
        $sut = $this->getMockBuilder(SensitiveFilterProcessor::class)
            ->onlyMethods(['convertStringsToRegex'])
            ->disableOriginalConstructor()
            ->getMock();
        $sut->expects($this->once())->method('convertStringsToRegex');

        $this->callMethod(
            $sut,
            '__construct',
            [['foo'], $replacementArgument]
        );

        $this->assertSame(
            $expectedReplacement,
            $this->getValue(
                $sut,
                'replacement'
            )
        );
    }

    public static function constructDataProvider(): Generator
    {
        yield 'default replacement' => [null, '*****'];
    }

    /**
     * @test
     * @throws ReflectionException
     * @dataProvider convertStringsToRegexDataProvider
     * @covers       \D3\LoggerFactory\SensitiveFilterProcessor::convertStringsToRegex
     * @covers       \D3\LoggerFactory\SensitiveFilterProcessor::stringIsRegexp
     */
    public function testConvertStringsToRegex(
        array $input,
        array $expected
    ): void {
        $sutMock = $this->getMockBuilder(SensitiveFilterProcessor::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->callMethod(
            $sutMock,
            'convertStringsToRegex',
            [$input]
        );

        $this->assertSame(
            $expected,
            $this->getValue(
                $sutMock,
                'secrets'
            )
        );
    }

    public static function convertStringsToRegexDataProvider(): Generator
    {
        yield 'simple' => [['abc'], ['/abc/i']];
        yield 'multiple' => [['def', 'ghi'], ['/def/i', '/ghi/i']];
        yield 'urlencoded' => [['1&c'], ['/1&c/i', '/1%26c/i']];
        yield 'delimiter' => [['de/fg'], ['/de\/fg/i', '/de%2Ffg/i']];
        yield 'regex' => [['/abc/mi'], ['/abc/mi']];
        yield 'mixed' => [['/abc/mi', 'def', '/ghi/mi'], ['/abc/mi', '/def/i', '/ghi/mi']];
    }

    /**
     * @test
     * @return void
     * @throws ReflectionException
     * @covers \D3\LoggerFactory\SensitiveFilterProcessor::__invoke
     */
    public function testInvoke()
    {
        $sut = new SensitiveFilterProcessor(['secret1', 'password'], '###');

        $record = [
            'key1'   => null,
            'key2'   => 'value+secret1+value+password',
            'password'  => 'secret1',
            'key4'  => [
                'subkey1' => null,
                'subkey2' => 'value+password+value+secret1',
                'subkey3' => [
                    'subsubkey3' => 'value+password+value+secret1',
                ],
            ],
        ];

        $expected = [
            'key1'   => null,
            'key2'   => 'value+###+value+###',
            'password'  => '###',
            'key4'  => [
                'subkey1' => null,
                'subkey2' => 'value+###+value+###',
                'subkey3' => [
                    'subsubkey3' => 'value+###+value+###',
                ],
            ],
        ];

        $this->assertSame(
            $expected,
            $this->callMethod(
                $sut,
                '__invoke',
                [$record]
            )
        );
    }
}
